"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combineMiddlewareWithActionHandlers = combineMiddlewareWithActionHandlers;
exports.default = composeMiddleware;

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _forEach2 = _interopRequireDefault(require("lodash/forEach"));

var _size2 = _interopRequireDefault(require("lodash/size"));

var _createMiddleware = _interopRequireDefault(require("./createMiddleware"));

var _checkForNonFunctions = _interopRequireDefault(require("./checkForNonFunctions"));

var _addCreationStack = _interopRequireDefault(require("./addCreationStack"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param middlewares an array of one or more middlewares.
 * @returns a middleware that calls all of the given middlewares in sequence.
 */
function naiveComposeMiddleware(middlewares) {
  if (middlewares.length === 0) return store => dispatch => dispatch;
  if (middlewares.length === 1) return middlewares[0];
  return store => next => {
    let handleAction = middlewares.reduceRight((next, handler) => handler(store)(next), next);
    if (process.env.NODE_ENV !== 'production') handleAction = (0, _addCreationStack.default)(handleAction, 'middleware');
    return handleAction;
  };
}

function composeActionHandlers(actionHandlers) {
  return (0, _createMiddleware.default)((0, _mapValues2.default)(actionHandlers, naiveComposeMiddleware));
}

function combineMiddlewareWithActionHandlers(...middlewares) {
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions.default)(middlewares, 'middlewares');
  const optimizedMiddlewares = [];
  let actionHandlers = {};
  middlewares.forEach(middleware => {
    if (middleware.actionHandlers) {
      (0, _forEach2.default)(middleware.actionHandlers, (actionHandler, type) => {
        ;
        (actionHandlers[type] || (actionHandlers[type] = [])).push(actionHandler);
      });
    } else {
      if ((0, _size2.default)(actionHandlers)) {
        optimizedMiddlewares.push(composeActionHandlers(actionHandlers));
        actionHandlers = {};
      }

      optimizedMiddlewares.push(middleware);
    }
  });
  if ((0, _size2.default)(actionHandlers)) optimizedMiddlewares.push(composeActionHandlers(actionHandlers));
  return optimizedMiddlewares;
}

function composeMiddleware(...middlewares) {
  return naiveComposeMiddleware(combineMiddlewareWithActionHandlers(...middlewares));
}