"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combineReducersWithActionHandlers = combineReducersWithActionHandlers;
exports.default = composeReducers;

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _forEach2 = _interopRequireDefault(require("lodash/forEach"));

var _size2 = _interopRequireDefault(require("lodash/size"));

var _createReducer = _interopRequireDefault(require("./createReducer"));

var _addCreationStack = _interopRequireDefault(require("./addCreationStack"));

var _checkForNonFunctions = _interopRequireDefault(require("./checkForNonFunctions"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param reducers an array of one or more reducers.
 * @returns a reducer that calls all of the given reducers in sequence.
 */
function naiveComposeReducers(reducers) {
  if (reducers.length === 0) return state => state;
  if (reducers.length === 1) return reducers[0];
  const initialState = reducers.reduce((state, reducer) => reducer(state, {
    type: ''
  }), undefined);
  return (state = initialState, action) => reducers.reduce((state, reducer) => reducer(state, action), state);
}

function composeActionHandlers(initialState, actionHandlers) {
  return (0, _createReducer.default)(initialState, (0, _mapValues2.default)(actionHandlers, naiveComposeReducers));
}

function combineReducersWithActionHandlers(...reducers) {
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions.default)(reducers, 'reducers');
  const optimizedReducers = [];
  let initialState = undefined;
  let actionHandlers = {};
  reducers.forEach(reducer => {
    if (reducer.actionHandlers) {
      initialState = reducer(initialState, {
        type: ''
      });
      (0, _forEach2.default)(reducer.actionHandlers, (actionHandler, type) => {
        ;
        (actionHandlers[type] || (actionHandlers[type] = [])).push(actionHandler);
      });
    } else {
      if ((0, _size2.default)(actionHandlers)) {
        optimizedReducers.push(composeActionHandlers(initialState, actionHandlers));
        actionHandlers = {};
        initialState = undefined;
      }

      optimizedReducers.push(reducer);
    }
  });
  if ((0, _size2.default)(actionHandlers)) optimizedReducers.push(composeActionHandlers(initialState, actionHandlers));
  return optimizedReducers;
}

function composeReducers(...reducers) {
  let result = naiveComposeReducers(combineReducersWithActionHandlers(...reducers));
  if (process.env.NODE_ENV !== 'production') result = (0, _addCreationStack.default)(result, 'reducer');
  return result;
}