"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPluggableMiddleware;

var _memoize2 = _interopRequireDefault(require("lodash/memoize"));

var _checkForNonFunctions = _interopRequireDefault(require("./checkForNonFunctions"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This is used for hot reloading redux middleware.
 *
 * let hmrMiddleware = createPluggableMiddleware(myMiddleware);
 * // hmrMiddleware can be used with applyMiddleware() just like anything else.
 * // but later if you need to replace it you can do:
 * hmrMiddleware.replaceMiddleware(myNewMiddleware);
 */
function createPluggableMiddleware(middleware) {
  if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions.default)(middleware, 'middleware');

  const result = store => next => {
    const dispatch = (0, _memoize2.default)(middleware => middleware ? middleware(store)(next) : next);
    return action => dispatch(middleware)(action);
  };

  result.replaceMiddleware = nextMiddleware => {
    if (process.env.NODE_ENV !== 'production') (0, _checkForNonFunctions.default)(nextMiddleware, 'nextMiddleware');
    middleware = nextMiddleware;
  };

  return result;
}